@@ -84,10 +84,10 @@ try_download <- function(from_url, to_file, hush = quietly) {
     return(FALSE)
   }
   # We download some fairly large files, so ensure the timeout is set appropriately.
-  # This assumes a static library size of 100 MB (generous) and a download speed
-  # of .3 MB/s (slow). This is to anticipate slower user connections or load on
-  # artifactory servers.
-  opts <- options(timeout = max(300, getOption("timeout")))
+  # This assumes a static library size of 100 MB (our current biggest is 78 MB) and
+  # a download speed of 0.2 MB/s (slow). This is to anticipate slower user connections
+  # or load on artifactory servers.
+  opts <- options(timeout = max(600, getOption("timeout")))
   on.exit(options(opts))
 
   status <- try(
@@ -926,7 +926,9 @@ options(.arrow.cleanup = character())
 on.exit(unlink(getOption(".arrow.cleanup"), recursive = TRUE), add = TRUE)
 
 not_cran <- env_is("NOT_CRAN", "true")
-if (not_cran) {
+on_r_universe <- !env_is("MY_UNIVERSE", "")
+
+if (not_cran || on_r_universe) {
   # Set more eager defaults
   if (env_is("LIBARROW_BINARY", "")) {
     Sys.setenv(LIBARROW_BINARY = "true")
